#ifndef HEPVis_SoXtHepEventViewer_H
#define HEPVis_SoXtHepEventViewer_H

#include <Xm/PushB.h>

#include <HEPVis/Xt/viewers/SoXtHepViewer.h>
#include <vector>
#include <list>

class SoPath;
class SoNode;
class SoGroup;
class SoSwitch;

typedef int (*EventSelectCallbackProc) (int, int, void *);
typedef int (*FileOpenCallbackProc)    (char *, void *);
typedef int (*NextEventCallbackProc)   (void *);
typedef int (*RewindCallbackProc)      (void *);

class SoXtHepEventViewer: public SoXtHepViewer
{
  
public:

  // Constructor
  SoXtHepEventViewer(Widget parent=NULL, 
		     const char *name="HEPVis Event Viewer", 
		     SbBool buildInsideParent=TRUE);
  // Destructor
  virtual ~SoXtHepEventViewer();
  
  // Callback Functions
  virtual void setEventSelectCallback(EventSelectCallbackProc proceedure, void *call_data=NULL);
  virtual void setNextEventCallback  (NextEventCallbackProc   proceedure, void *call_data=NULL);
  virtual void setRewindCallback     (RewindCallbackProc      proceedure, void *call_data=NULL);
  virtual void setFileOpenCallback   (FileOpenCallbackProc    proceedure, void *call_data=NULL);

  // Set run and event number
  virtual void setRunNumber(int);
  virtual void setEventNumber(int);

  // Iconize something:
  virtual void iconize (SoPath *pathToNode, const char *name);

  class Icon {
    
  public:

    SoNode   *node;
    SoGroup  *parent;
    int       index;
    SoSwitch *sw;
    
    bool operator < (const Icon & other) const {
      return node < other.node && parent < other.parent && index < other.index;
    }

    bool operator == (const Icon & other) const {
      return node == other.node && parent == other.parent && index == other.index;
    }
    bool operator != (const Icon & other) const {
      return !operator==(other);
    }
		     
  };


protected:
  
  // These are for the file dialog...
  static void openCallback(Widget w, XtPointer callData, XmPushButtonCallbackStruct *callbackStruct);
  static void invokeFileOpen(Widget, void *, XmFileSelectionBoxCallbackStruct *);  

  // These are for event filtering
  static void invokeSelectEvent (Widget w, XtPointer callData, XmAnyCallbackStruct *callbackStruct);
  static void invokeNextEvent(Widget w, XtPointer callData, XmPushButtonCallbackStruct *callbackStruct);
  static void invokeRewind(Widget w, XtPointer callData, XmPushButtonCallbackStruct *callbackStruct);
  static void deiconize(Widget w, XtPointer callData, XmListCallbackStruct *callbackStruct);

  virtual void create(Widget);
  
  void clear();

  Widget _filterForm;


  Widget _eventLabel;
  Widget _runLabel;
  Widget _eventNumberField;
  Widget _runNumberField;
  Widget _nextEventButton;
  Widget _rewindButton;
  Widget _theIconBox;
  //  Widget _openButton;
  
private: 
  
  static String         _defaultSoXtHepEventViewerResources[];
  
  // Callbacks
  
  EventSelectCallbackProc _eventSelectCallback;
  FileOpenCallbackProc    _fileOpenCallback;
  NextEventCallbackProc   _nextEventCallback;
  RewindCallbackProc      _rewindCallback;

  // Call data

  void *_eventSelectCallData;
  void *_fileOpenCallData;
  void *_nextEventCallData;
  void *_rewindCallData;

  std::list<Icon>                          _iconList;
  std::vector<std::list<Icon>::iterator>  _activeIcons;

};

#endif


