#ifndef Lib_Message_h
#define Lib_Message_h

#include <Slash/Core/IMessage.h>

namespace Slash {namespace Core {class IAction;}}
namespace Slash {namespace Core {class IRelation;}}

namespace Lib {

class BaseMessage : public virtual Slash::Core::IMessage {
public:
  BaseMessage();
  virtual ~BaseMessage();
};

class ExitMessage : public BaseMessage {
public:
  virtual void* cast(const std::string&) const;
  virtual std::string message() const;
public:
  ExitMessage(int = 0);
  virtual ~ExitMessage();
  int code() const;
private:
  int fCode;
};

class CreateMessage : public BaseMessage {
public:
  virtual void* cast(const std::string&) const;
  virtual std::string message() const;
public:
  CreateMessage(const std::string& = "");
  virtual ~CreateMessage();
  const std::string& what() const;
private:
  std::string fWhat;
};

class DeleteMessage : public BaseMessage {
public:
  virtual void* cast(const std::string&) const;
  virtual std::string message() const;
public:
  DeleteMessage(const std::string& = "");
  virtual ~DeleteMessage();
  const std::string& what() const;
private:
  std::string fWhat;
};

class ExecuteMessage : public BaseMessage {
public:
  virtual void* cast(const std::string&) const;
  virtual std::string message() const;
public:
  ExecuteMessage(Slash::Core::IAction&);
  virtual ~ExecuteMessage();
  Slash::Core::IAction& action() const;
private:
  Slash::Core::IAction& fAction;
};

class RelationMessage : public BaseMessage {
public: //IMessage:
  virtual void* cast(const std::string&) const;
  virtual std::string message() const;
public:
  RelationMessage(Slash::Core::IRelation&);
  virtual ~RelationMessage();
  Slash::Core::IRelation& relation() const;
private:
  Slash::Core::IRelation& fRelation;
};

class UpdateMessage : public BaseMessage {
public:
  virtual void* cast(const std::string&) const;
  virtual std::string message() const;
public:
  UpdateMessage(const std::string& = "",Slash::Core::IRelation* = 0);
  virtual ~UpdateMessage();
  const std::string& what() const;
  Slash::Core::IRelation* relation() const;
private:
  std::string fWhat;
  Slash::Core::IRelation* fRelation;
};

}

#endif
