//
//  Deprecated. In principle Variable is no more used
// in OSC-16.1.
//

#ifndef Lib_Variable_h
#define Lib_Variable_h

namespace Slash {namespace Core {class IWriter;}}
namespace Slash {namespace Core {class IValue;}}

#include <string>

#include <Lib/Array.h>
#include <Lib/typedefs.h>

namespace Lib {

class Variable {
public: // Internal :
  enum eType {
    NONE = 0,
    INTEGER = 1,
    DOUBLE = 2,
    STRING = 3,
    POINTER = 4,
    UNSIGNED_INTEGER = 5,
    BOOLEAN = 6,
    SHORT = 7,
    FLOAT = 9,
    UNSIGNED_CHAR = 10,
    CHAR = 11,
    VECTOR_DOUBLE = 12,
    INT64 = 14,
    UNSIGNED_INT64 = 15,
    UNSIGNED_SHORT = 16,
    DOUBLE_STAR = 101,
    FLOAT_STAR = 102,
    INT_STAR = 103
  };
  static std::string stype(eType);
public:
  Variable(Slash::Core::IWriter&);
  Variable(const Variable&);
  virtual ~Variable();
  // By value :
  Variable(Slash::Core::IWriter&,bool);
  Variable(Slash::Core::IWriter&,char);
  Variable(Slash::Core::IWriter&,short);
  Variable(Slash::Core::IWriter&,int);
  Variable(Slash::Core::IWriter&,Lib::int64);
  Variable(Slash::Core::IWriter&,float);
  Variable(Slash::Core::IWriter&,double);
  Variable(Slash::Core::IWriter&,unsigned char);
  Variable(Slash::Core::IWriter&,unsigned short);
  Variable(Slash::Core::IWriter&,unsigned int);
  Variable(Slash::Core::IWriter&,Lib::uint64);
  Variable(Slash::Core::IWriter&,const char*);
  Variable(Slash::Core::IWriter&,const std::string&);
  Variable(Slash::Core::IWriter&,void*);
  Variable(Slash::Core::IWriter&,const Array<double>&);
  Variable(Slash::Core::IWriter&,double*);
  Variable(Slash::Core::IWriter&,float*);
  Variable(Slash::Core::IWriter&,int*);

  // Else
  Variable& operator =(const Variable&);
  //void print(Slash::Core::IWriter&) const;

  std::string toString() const;

  bool to_double(double&);
  //bool to_bool(bool&)) const;

  void setName(const std::string&);
  std::string name() const;
  void setDimension(unsigned int);
  unsigned int dimension() const;

  eType type() const;
  void setType(eType);
  void setNone();
  std::string stype() const;
  
  bool is_none() const;
  bool is_bool() const;
  bool is_double() const;
  bool is_vector_double() const;

  // Used in BatchLab/<IO>_Tuple.cxx.
  unsigned char get_unsigned_char() const;
  char get_char() const;
  unsigned short get_unsigned_short() const;
  unsigned int get_unsigned_int() const;
  int get_int() const;
  Lib::int64 get_int64() const;
  short get_short() const;
  float get_float() const;
  double get_double() const;
  void* get_void_star() const;
  double* get_double_star() const;
  float* get_float_star() const;
  int* get_int_star() const;
  bool get_bool() const;
  std::string get_std_string() const;
  //Used in OnXLab/[Histogram,Tuple]Action.cxx and 
  // OpenPAW/vector_C.cxx and pasigm.cxx :
  Array<double>& get_Array_double();
  
  void set(unsigned char);
  void set(char);
  void set(unsigned short);
  void set(unsigned int);
  void set(Lib::uint64);
  void set(int);
  void set(Lib::int64);
  void set(short);
  void set(float);
  void set(double);
  void set(bool);
  void set(const std::string&);
  void set(const char*);
  void set(const Array<double>&);
  void set(void*);
  void set(double*);
  void set(float*);
  void set(int*);

  bool set(const Slash::Core::IValue&);

  // Used in Columns.y :
  bool assign(const Variable&,std::string&);

  // operations in Expression.cxx :
  bool minus(std::string&);
  bool do_not(std::string&);

  bool add(const Variable&,std::string&);
  bool subtract(const Variable&,std::string&);
  bool multiply(const Variable&,std::string&);
  bool divide(const Variable&,std::string&);
    
  bool if_gt(const Variable&,std::string&);
  bool if_eq(const Variable&,std::string&);
  bool if_ne(const Variable&,std::string&);
  bool if_ge(const Variable&,std::string&);
  bool if_lt(const Variable&,std::string&);
  bool if_le(const Variable&,std::string&);
  bool if_and(const Variable&,std::string&);
  bool if_or(const Variable&,std::string&);

  bool cxx_type(std::string&) const;
public:
  static Slash::Core::IValue* toValue(const Lib::Variable&);
private:
  void reset();
private:
  Slash::Core::IWriter& fPrinter;
  std::string* fName;
  unsigned int fDimension;
private:
  eType fType;
  union {
    bool f_bool;
    char f_char;
    int f_int;
    short f_short;
    Lib::int64 f_int64;
    float f_float;
    double f_double;
    unsigned char f_unsigned_char;
    unsigned int f_unsigned_short;
    unsigned int f_unsigned_int;
    Lib::uint64 f_unsigned_int64;
    void* f_void_star;
    double* f_double_star;
    float* f_float_star;
    int* f_int_star;
    std::string* f_std_string;
    Array<double>* f_Array_double;
  } u;
};

}

#define Lib_Variable_error_div_zero "Lib::Variable::divide : divide by zero."

#endif



