#ifndef Lib_vmanip_h
#define Lib_vmanip_h

#include <vector>

namespace Lib {

class vmanip {
public:
  static int binarySearch(const std::vector<double>&,double);
  static std::vector<float> convert(const std::vector<double>&);
};
              
template <class T>
inline void append(std::vector<T>& aVector,const std::vector<T>& aWhat) {
  unsigned int vsize = aVector.size();
  unsigned int number = aWhat.size();
  aVector.resize(vsize+number);
  unsigned offset = vsize; 
  for(unsigned int index=0;index<number;index++,offset++) {
    aVector[offset] = aWhat[index];
  }
}

// From :
//   http://softsurfer.com/Archive/algorithm_0103/algorithm_0103.htm


// Copyright 2001, softSurfer (www.softsurfer.com)
// This code may be freely used and modified for any purpose
// providing that this copyright notice is included with it.
// SoftSurfer makes no warranty for this code, and cannot be held
// liable for any real or imagined damage resulting from its use.
// Users of this code must verify correctness for their application.

template <class TP,class T>
inline TP is_left(const T& P0,const T& P1,const T& P2){
  return ( (P1[0] - P0[0]) * (P2[1] - P0[1])
         - (P2[0] - P0[0]) * (P1[1] - P0[1]) );
}


//               V[] = vertex points of a polygon V[n+1] with V[n]=V[0]
template <class TP,class T>
bool inside(const T& a_P,const std::vector<T>& a_V,bool& a_is) {

  if(!a_V.size()) return false;
  unsigned int n = a_V.size()-1;
  if(a_V[n]!=a_V[0]) return false;

  int    wn = 0;    // the winding number counter

  // loop through all edges of the polygon
  for (unsigned int i=0; i<n; i++) {   // edge from V[i] to V[i+1]
    if (a_V[i][1] <= a_P[1]) {         // start y <= P[1]
      if (a_V[i+1][1] > a_P[1])      // an upward crossing
        if (is_left<TP,T>( a_V[i], a_V[i+1], a_P) > 0)  // P left of edge
          ++wn;            // have a valid up intersect
    } else {                       // start y > P[1] (no test needed)
      if (a_V[i+1][1] <= a_P[1])     // a downward crossing
        if (is_left<TP,T>( a_V[i], a_V[i+1], a_P) < 0)  // P right of edge
          --wn;            // have a valid down intersect
    }
  }

  a_is = ((wn!=0)?true:false);

  return true;
}

}

#endif
