#ifndef OnX_NextStepTk_h
#define OnX_NextStepTk_h 

#import <AppKit/NSButton.h>
#import <AppKit/NSMenuItem.h>
#import <AppKit/NSPopUpButton.h>
#import <AppKit/NSToolbar.h>
#import <AppKit/NSToolbarItem.h>
#import <AppKit/NSView.h>
#import <AppKit/NSOpenGLView.h>
#import <AppKit/NSOutlineView.h>
#import <AppKit/NSEvent.h>
#import <AppKit/NSTextField.h>
#import <AppKit/NSComboBox.h>
#import <AppKit/NSSecureTextField.h>
#import <AppKit/NSSlider.h>
#import <AppKit/NSScroller.h>
#import <AppKit/NSTabView.h>
#import <AppKit/NSSplitView.h>
#import <AppKit/NSWindow.h>

@class NSOpenPanel;
@class NSSavePanel;
@class NSColorPanel;
@class OnX_NSAction;

@interface OnX_NSButton : NSButton {
  NSMutableArray* fActivateActions;
}
- (id)initWithFrame:(NSRect)aRect;
- (void)dealloc;
- (void)addActivateAction:(OnX_NSAction*)aAction;
- (void)action:(id)aSender;
@end

@interface OnX_NSToggleButton : OnX_NSButton {
}
- (id)initWithFrame:(NSRect)aRect;
- (void)dealloc;
- (void)action:(id)aSender;
@end

@interface OnX_NSMenuItem : NSMenuItem {
  NSMutableArray* fActivateActions;
}
- (id)initWithTitle:(NSString*)aString action:(SEL)aSelector keyEquivalent:(NSString*)aCharCode;
- (void)dealloc;
- (void)addActivateAction:(OnX_NSAction*)aAction;
- (void)action:(id)aSender;
@end

@interface OnX_NSToggleMenuItem : OnX_NSMenuItem {
}
- (id)initWithTitle:(NSString*)aString action:(SEL)aSelector keyEquivalent:(NSString*)aCharCode;
- (void)dealloc;
- (void)action:(id)aSender;
@end

@interface OnX_NSPopUpButton : NSPopUpButton {
  NSMutableArray* fActivateActions;
}
- (id)initWithFrame:(NSRect)aRect;
- (void)dealloc;
- (void)addActivateAction:(OnX_NSAction*)aAction;
- (void)action:(id)aSender;
@end

@interface OnX_NSTextField : NSTextField {
  NSMutableArray* fActivateActions;
  NSMutableArray* fCompleteActions;
}
- (id)initWithFrame:(NSRect)aRect;
- (void)dealloc;
- (void)addActivateAction:(OnX_NSAction*)aAction;
- (void)addCompleteAction:(OnX_NSAction*)aAction;
- (void)action:(id)aSender;
@end

@interface OnX_NSSecureTextField : NSSecureTextField {
  NSMutableArray* fActivateActions;
}
- (id)initWithFrame:(NSRect)aRect;
- (void)dealloc;
- (void)addActivateAction:(OnX_NSAction*)aAction;
- (void)action:(id)aSender;
@end

@interface OnX_NSComboBox : NSComboBox {
  NSMutableArray* fActivateActions;
}
- (id)initWithFrame:(NSRect)aRect;
- (void)dealloc;
- (void)addActivateAction:(OnX_NSAction*)aAction;
- (void)action:(id)aSender;
@end

@interface OnX_NSToolbarItem : NSToolbarItem {
  NSMutableArray* fActivateActions;
}
- (id)initWithItemIdentifier:(NSString*)aData;
- (void)dealloc;
- (void)addActivateAction:(OnX_NSAction*)aAction;
- (void)action:(id)aSender;
@end

@interface OnX_NSToolbar : NSToolbar {
  NSMutableArray* fItems;
}
- (id)initWithIdentifier:(NSString*)aData;
- (void)dealloc;
- (void)addToolbarItem:(NSToolbarItem*)aItem;
@end

typedef enum _OnX_Expand {
  OnX_FIRST = 0,
  OnX_SECOND = 1,
  OnX_BOTH = 3
} OnX_Expand;

@interface OnX_NSVBox : NSView {
  OnX_Expand fExpand;  
  int fSizeOfFixed;
}
- (id)initWithFrame:(NSRect)aRect;
- (void)dealloc;
- (void)setExpand:(OnX_Expand)aExpand;
- (void)addView:(NSView*)aView;
- (void)setSizeOfFixed:(int)aPosition;
@end

@interface OnX_NSHBox : NSView {
  OnX_Expand fExpand;  
  int fSizeOfFixed;
}
- (id)initWithFrame:(NSRect)aRect;
- (void)dealloc;
- (void)setExpand:(OnX_Expand)aExpand;
- (void)addView:(NSView*)aView;
- (void)setSizeOfFixed:(int)aPosition;
@end

@interface OnX_NSVPaned : NSSplitView {
  int fSplitPosition;
}
- (id)initWithFrame:(NSRect)aRect;
- (void)dealloc;
- (void)addView:(NSView*)aView;
- (void)setSplitPosition:(int)aPosition;
@end

@interface OnX_NSHPaned : NSSplitView {
  int fSplitPosition;
}
- (id)initWithFrame:(NSRect)aRect;
- (void)dealloc;
- (void)addView:(NSView*)aView;
- (void)setSplitPosition:(int)aPosition;
@end

@interface OnX_NSVContainer : NSView {
}
- (id)init;
- (void)dealloc;
- (void)addView:(NSView*)aView;
@end

@interface OnX_NSHContainer : NSView {
}
- (id)init;
- (void)dealloc;
- (void)addView:(NSView*)aView;
@end

@interface OnX_NSOpenGLArea : NSOpenGLView {
  NSMutableArray* fPaintActions;
  NSMutableArray* fEventActions;
  NSEventType fEventType;
  NSPoint fEventLocation;
}
- (id)initWithFrame:(NSRect)aRect;
- (void)dealloc;
- (void)addPaintAction:(OnX_NSAction*)aAction;
- (void)addEventAction:(OnX_NSAction*)aAction;
- (NSEventType)eventType;
- (NSPoint)eventLocation;
- (void)drawRect:(NSRect)aRect;
- (void)refresh;
- (NSOpenGLPixelFormat*)createPixelFormat:(NSRect)aFrame;
- (void)mouseDown:(NSEvent*)aEvent;
- (void)mouseUp:(NSEvent*)aEvent;
- (void)mouseDragged:(NSEvent*)aEvent;
@end

@interface OnX_NSFileChooser : NSObject {
  bool fModeOpen;
  NSOpenPanel* fOpenPanel;
  NSSavePanel* fSavePanel;
  NSMutableArray* fOkActions;
  NSMutableArray* fCancelActions;
  NSString* fFileName;
}
- (id)init;
- (void)dealloc;
- (void)addOkAction:(OnX_NSAction*)aAction;
- (void)addCancelAction:(OnX_NSAction*)aAction;
- (void)show;
- (void)openPanelDidEnd:(NSOpenPanel*)aPanel
             returnCode:(int)aRC 
            contextInfo:(void*)aContext;
- (NSString*)fileName;
- (void)setModeOpen:(bool)aMode;
@end

@interface OnX_NSColorChooser : NSObject {
  NSColorPanel* fColorPanel;
  NSMutableArray* fOkActions;
  NSString* fColor;
}
- (id)init;
- (void)dealloc;
- (void)addOkAction:(OnX_NSAction*)aAction;
- (void)show:(id)aSender;
- (void)hide;
- (NSString*)choosenColor;
- (void)changeColor:(id)aSender;
@end

@interface OnX_NSTreeItem : NSObject {
  OnX_NSTreeItem* fParent;
  NSMutableArray* fChildren;
  NSString* fLabel;
}
- (id)initWithParent:(OnX_NSTreeItem*)aParent;
- (void)dealloc;
- (void)addObject:(OnX_NSTreeItem*)aItem;
- (int)numberOfChildren;
- (OnX_NSTreeItem*)childAtIndex:(int)aIndex;
- (NSString*)label;
- (void)setLabel:(NSString*)aLabel;
- (NSString*)path;
@end

@interface OnX_NSTree : NSOutlineView {
  OnX_NSTreeItem* fRootItem;
  NSMutableArray* fSelectActions;
  NSString* fSelectedPath;
}
- (id)initWithFrame:(NSRect)aRect;
- (void)dealloc;
- (void)setRootItem:(OnX_NSTreeItem*)aItem;
- (void)addSelectAction:(OnX_NSAction*)aAction;
- (NSString*)selectedPath;
@end

@interface OnX_NSList : NSTableView {
  OnX_NSTreeItem* fRootItem;
  NSMutableArray* fSelectActions;
  NSString* fSelection;
}
- (id)initWithFrame:(NSRect)aRect;
- (void)dealloc;
- (void)setRootItem:(OnX_NSTreeItem*)aItem;
- (void)addSelectAction:(OnX_NSAction*)aAction;
- (NSString*)selection;
@end

@interface OnX_NSSlider : NSSlider {
  NSMutableArray* fActivateActions;
}
- (id)initWithFrame:(NSRect)aRect;
- (void)dealloc;
- (void)addActivateAction:(OnX_NSAction*)aAction;
- (void)action:(id)aSender;
@end

@interface OnX_NSScroller : NSScroller {
  NSMutableArray* fActivateActions;
}
- (id)initWithFrame:(NSRect)aRect;
- (void)dealloc;
- (void)addActivateAction:(OnX_NSAction*)aAction;
- (void)action:(id)aSender;
@end

@interface OnX_NSTabView : NSTabView {
  NSMutableArray* fSelectActions;
}
- (id)initWithFrame:(NSRect)aRect;
- (void)dealloc;
- (void)addSelectAction:(OnX_NSAction*)aAction;
@end

@interface OnX_NSDialog : NSWindow {
}
- (id)initWithContentRect:(NSRect)aRect;
- (void)dealloc;
@end

#endif
