#ifndef OnX_QTk_h
#define OnX_QTk_h 

// Inheritance :
#include <qsplitter.h>
#include <qgl.h>
#include <qdialog.h>
#include <qmainwindow.h>

#include <qevent.h>
class QCallbackData;

#if QT_VERSION < 0x040000
class QPopupMenu;
#else
class OnX_Q3PopupMenu;
#define QPopupMenu OnX_Q3PopupMenu
#endif

class OnX_QOpenGLArea : public QGLWidget {
  Q_OBJECT
public:
  OnX_QOpenGLArea(QWidget*,const char*);
public:
  virtual ~OnX_QOpenGLArea();
  virtual void initialieGL();
  virtual void resizeGL(int,int);
  virtual void paintGL();
  virtual void mousePressEvent(QMouseEvent*);
  virtual void mouseReleaseEvent(QMouseEvent*);
  virtual void mouseMoveEvent(QMouseEvent*);
signals:
  void paint(QCallbackData&);
  void eventHandler(QCallbackData&);
public:
  bool write_gl2ps(const std::string&,const std::string&); 
};

// To handle splitPosition at startup.
class OnX_QSplitter : public QSplitter {
  Q_OBJECT
public:
  OnX_QSplitter(Qt::Orientation,QWidget*,const char*);
  virtual ~OnX_QSplitter();
  void setSplitPosition(int);
  virtual void showEvent(QShowEvent*);
private:
  int fSplitPosition;
};

//WARNING : can't be put in the main panel. The show 
// on the main panel will show it and then we will
// be blocked in the QColorDialog::getColor().

//class OnX_QColorDialog : public QColorDialog {
class OnX_QColorDialog : public QWidget {
  Q_OBJECT
public:
#if QT_VERSION < 0x040000
  virtual void show();
  virtual void hide();
#else
  void map();
  void unmap();
#endif
public:
  OnX_QColorDialog(const char*);
  virtual ~OnX_QColorDialog();
  std::string value() const;
signals:
  void ok();
private:
  std::string fValue;
};

// Have a QWidget to handle the "save" case.
// Used only with Qt4.
//class OnX_QFileDialog : public QWidget {
class OnX_QFileDialog : public QDialog {
  Q_OBJECT
public:
  void map();
  void unmap();
public:
  OnX_QFileDialog(const char*);
  virtual ~OnX_QFileDialog();
  std::string value() const;
  enum AcceptMode { AcceptOpen, AcceptSave };
  void setAcceptMode(AcceptMode);
  void setFilter(const std::string&);
  void setDirectory(const std::string&);
signals:
  void ok();
private:
  AcceptMode fMode;
  std::string fFilter;
  std::string fDirectory;
  std::string fValue;
};

class OnX_QMenu : public QWidget {
  Q_OBJECT
public:
  OnX_QMenu(const char*,QPopupMenu&);
  virtual ~OnX_QMenu();
  QPopupMenu& popupMenu() const;
private:
  QPopupMenu& fPopup;
};

class OnX_QMenuItem : public QWidget {
  Q_OBJECT
public:
  OnX_QMenuItem(const char*,QPopupMenu&);
  virtual ~OnX_QMenuItem();
  QPopupMenu& popupMenu() const;
  int id() const;
  void set(const std::string&);
  void setAccel(const std::string&);
  bool label(std::string&) const;
  bool setLabel(const std::string&);
signals:
  void activated();
public slots:
  void execute(); //the popup activates it.
protected:
  QPopupMenu& fPopup;
  int fID;
};

class OnX_QToggleMenuItem : public OnX_QMenuItem {
  Q_OBJECT
public:
  OnX_QToggleMenuItem(const char*,QPopupMenu&);
  virtual ~OnX_QToggleMenuItem();
public:
  bool isChecked(bool&) const;
  bool setChecked(bool);
};

class OnX_QMainWindow : public QMainWindow {
  Q_OBJECT
public:
  OnX_QMainWindow(QWidget*,const char*,Qt::WFlags);
  virtual ~OnX_QMainWindow();
protected:
  virtual void closeEvent(QCloseEvent*);
signals:
  void close_window(); //can't be close. It clashes with QWidget::close.
};


#if QT_VERSION < 0x040000
#else
#undef QPopupMenu
#endif

#endif
