#ifndef Slash_Tools_cast_h
#define Slash_Tools_cast_h

#include <string>

namespace Slash {

template <class aT,class aF>
inline aT* cast(aF& a_o,const std::string& aCast) {
#if defined(__linux)
  // Python shell (2.3.3) looses the dynamic_cast on the DLL modules !
  // Then we stay with our "safe cast".
  return (aT*)a_o.cast(aCast);
#else
  return dynamic_cast<aT*>(&a_o);
#endif
}

template <class aT,class aF>
inline const aT* cast_const(const aF& a_o,const std::string& aCast) {
#if defined(__linux)
  // Python shell (2.3.3) looses the dynamic_cast on the DLL modules !
  // Then we stay with our "safe cast".
  return (const aT*)a_o.cast(aCast);
#else
  return dynamic_cast<const aT*>(&a_o);
#endif
}

}

#if defined(__linux)
// Python shell (2.3.3) looses the dynamic_cast on the DLL modules !
// Then we stay with our "safe cast".
#define Slash_cast(a__obj,a__cls) (a__cls*)(a__obj).cast(#a__cls)
#define Slash_const_cast(a__obj,a__cls) (const a__cls*)(a__obj).cast(#a__cls)
#else
#define Slash_cast(a__obj,a__cls) dynamic_cast<a__cls*>(&a__obj)
#define Slash_const_cast(a__obj,a__cls) dynamic_cast<const a__cls*>(&a__obj)
#endif

#define Slash_safe_cast(a__obj,a__cls) (a__cls*)(a__obj).cast(#a__cls)

#endif
